#include <windows.h>
#include <vfw.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static const char * myname = "wvctest";	/* ̃AvP[V̖O */
static BOOL drvopen = FALSE; /* hCoI[vĂ邩 */
static HWND capwin = NULL; /* Lv`EBhẼnh */
static DWORD start = 0; /* Lv`Jn */
static DWORD frame = 0;	/* Lv`t[ */

/* G[bZ[W\Ĕ */
static void error (const char * message) {
  char buf [1024];

  sprintf (buf, "ERROR: %s: %x", message, GetLastError ());

  MessageBox (NULL,
	      buf,
	      myname,
	      MB_OK | MB_ICONINFORMATION | MB_DEFBUTTON1 | MB_APPLMODAL);

  if (drvopen)
    capDriverDisconnect (capwin);

  exit (1);
}

/* EBhE̔wiFԂ */
static HGDIOBJ getbg (void) {
  HGDIOBJ tmp;

  if (! (tmp = GetStockObject (WHITE_BRUSH)))
    error ("GetStockObject");

  return tmp;
}

static LRESULT CALLBACK winproc (HWND hwnd,
				 UINT uMsg,
				 WPARAM wParam,
				 LPARAM lParam) {
  switch (uMsg) {
  case WM_PAINT:
    /* ŜwiFœhԂ */
    {
      HDC hdc;
      PAINTSTRUCT ps;
      RECT rect;

      if (! (hdc = BeginPaint (hwnd, & ps)))
	error ("BeginPaint");

      if (! GetClientRect (hwnd, & rect))
	error ("GetClientRect");

      if (! FillRect (hdc, & rect, getbg ()))
	error ("FillRect");

      EndPaint (hwnd, & ps); /* This function always returns TRUE. */

      return 0;
    }
  case WM_KEYDOWN:
    /* t[[gvZĕ\ */
    {
      DWORD end;
      double sec;
      double fps;
      char buf [1024];

      end = timeGetTime ();
      sec = (double) (end - start) / 1000.0;
      fps = (double) frame / max (sec, 0.001);

      sprintf (buf, "%d frames / %.1f seconds = %.1f fps", frame, sec, fps);

      MessageBox (NULL,
		  buf,
		  myname,
		  MB_OK | MB_ICONINFORMATION | MB_DEFBUTTON1 | MB_APPLMODAL);

      start = timeGetTime ();
      frame = 0;

      return 0;
    }
  case WM_DESTROY:
    {
      if (drvopen)
	capDriverDisconnect (capwin);

      exit (0);
    }
  }

  return DefWindowProc (hwnd, uMsg, wParam, lParam);
}

int WINAPI WinMain (HINSTANCE hInstance,
		    HINSTANCE hPrevInstance,
		    LPSTR lpszCmdLine,
		    int nCmdShow) {
  HWND win;
  MSG msg;

  if (! hPrevInstance) {
    WNDCLASS class;

    class . style = CS_HREDRAW | CS_VREDRAW;
    class . lpfnWndProc = winproc;
    class . cbClsExtra = 0;
    class . cbWndExtra = 0;
    class . hInstance = hInstance;
    class . hIcon = LoadIcon (NULL, IDI_APPLICATION);
    class . hCursor = LoadCursor (NULL, IDC_ARROW);
    class . hbrBackground = getbg ();
    class . lpszMenuName = NULL;
    class . lpszClassName = myname;

    if (! RegisterClass (& class))
      error ("RegisterClass");
  }

  if (! (win = CreateWindow (myname,
			     myname,
			     WS_OVERLAPPEDWINDOW,
			     CW_USEDEFAULT,
			     CW_USEDEFAULT,
			     CW_USEDEFAULT,
			     CW_USEDEFAULT,
			     NULL,
			     NULL,
			     hInstance,
			     NULL)))
    error ("CreateWindow");

  /* Lv`EBhE쐬 */
  if (! (capwin = capCreateCaptureWindow (myname,
					  WS_CHILD | WS_VISIBLE,
					  0, 0,
					  640, 480,
					  win, 0)))
    error ("capCreateCaptureWindow");
  
  /* hCoI[v */
  if (! capDriverConnect (capwin, 0))
    error ("capDriverConnect");
  
  drvopen = TRUE;
  
  /* \[Xݒ_CA[O{bNX\ */
  if (! capDlgVideoSource (capwin))
    error ("capDlgVideoSource");
  
  /* tH[}bgݒ_CA[O{bNX\ */
  if (! capDlgVideoFormat (capwin))
    error ("capDlgVideoFormat");

  /* 摜̑傫ɍ킹ăEBhETCY */
  {
    int x, y, width, height;

    /* 摜̑傫擾 */
    {
      size_t len;
      LPBITMAPINFO format;

      if ((len = capGetVideoFormatSize (capwin)) <= 0)
	error ("capGetVideoFormatSize");

      if (! (format = malloc (len)))
	error ("malloc");

      if (capGetVideoFormat (capwin, format, len) != len) {
        free (format);
	error ("capGetVideoFormat");
      }

      width = format -> bmiHeader . biWidth;
      height = format -> bmiHeader . biHeight;

      free (format);
    }

    /* 摜̑傫ɍEBhȆ傫vZ */
    {
      RECT rect;

      rect . left = 0;
      rect . top = 0;
      rect . right = width;
      rect . bottom = height;

      if (! AdjustWindowRect (& rect, WS_OVERLAPPEDWINDOW, FALSE))
	error ("AdjustWindowRect");

      width = rect . right - rect . left;
      height = rect . bottom - rect . top;
    }

    /* ݂̃EBhËʒu擾 */
    {
      RECT rect;

      if (! GetWindowRect (win, & rect))
        error ("GetWindowRect");

      x = rect . left;
      y = rect . top;
    }

    if (! MoveWindow (win, x, y, width, height, TRUE))
      error ("MoveWindow");
  }

  if (! ShowWindow (win, nCmdShow))
    /* error ("ShowWindow") */;

  if (! UpdateWindow (win))
    error ("UpdateWindow");

  start = timeGetTime ();
  frame = 0;

  /* Lv`sbZ[W */
  while (TRUE) {
    if (! capGrabFrame (capwin))
      error ("capGrabFrame");

    frame ++;

    if (PeekMessage (& msg, NULL, 0, 0, PM_REMOVE))
      DispatchMessage (& msg);
  }

  return 0;
}
